package com.ruoyi.oa.mapper;

import java.util.List;
import com.ruoyi.oa.domain.CmcSettle;

/**
 * cmc结算审批Mapper接口
 * 
 * @author cmc
 * @date 2024-04-29
 */
public interface CmcSettleMapper 
{
    /**
     * 查询cmc结算审批
     * 
     * @param settleId cmc结算审批主键
     * @return cmc结算审批
     */
    public CmcSettle selectCmcSettleBySettleId(String settleId);

    /**
     * 查询cmc结算审批列表
     * 
     * @param cmcSettle cmc结算审批
     * @return cmc结算审批集合
     */
    public List<CmcSettle> selectCmcSettleList(CmcSettle cmcSettle);

    /**
     * 新增cmc结算审批
     * 
     * @param cmcSettle cmc结算审批
     * @return 结果
     */
    public int insertCmcSettle(CmcSettle cmcSettle);

    /**
     * 修改cmc结算审批
     * 
     * @param cmcSettle cmc结算审批
     * @return 结果
     */
    public int updateCmcSettle(CmcSettle cmcSettle);

    /**
     * 删除cmc结算审批
     * 
     * @param settleId cmc结算审批主键
     * @return 结果
     */
    public int deleteCmcSettleBySettleId(String settleId);

    /**
     * 批量删除cmc结算审批
     * 
     * @param settleIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteCmcSettleBySettleIds(String[] settleIds);
}
